@@ -27,6 +27,7 @@
 #include <mpiglue.h>
 #include <mpi_utils.h>
 #include <maxwell.h>
+#include <xyz_loop.h>
 
 #include <ctl-io.h>
 #include <ctlgeom.h>
@@ -67,13 +68,20 @@ void get_dfield(int which_band)
      curfield = (scalar_complex *) mdata->fft_data;
      curfield_band = which_band;
      curfield_type = 'd';
-     maxwell_compute_d_from_H(mdata, H, curfield, which_band - 1, 1);
+     if (mdata->mu_inv == NULL)
+         maxwell_compute_d_from_H(mdata, H, curfield, which_band - 1, 1);
+     else {
+         evectmatrix_resize(&W[0], 1, 0);
+         maxwell_compute_H_from_B(mdata, H, W[0], curfield, which_band-1,0, 1);
+         maxwell_compute_d_from_H(mdata, W[0], curfield, 0, 1);
+         evectmatrix_resize(&W[0], W[0].alloc_p, 0);
+     }
 
      /* Here, we correct for the fact that compute_d_from_H actually
 	computes just (k+G) x H, whereas the actual D field is
-	i/omega i(k+G) x H...so, there is an added factor of -1/omega. 
+	i/omega i(k+G) x H...so, there is an added factor of -1/omega.
 
-        We also divide by the cell volume so that the integral of |H|^2
+        We also divide by the cell volume so that the integral of H*B
         or of D*E is unity.  (From the eigensolver + FFT, they are
         initially normalized to sum to nx*ny*nz.) */
      {
@@ -117,12 +125,57 @@ void get_hfield(integer which_band)
      curfield = (scalar_complex *) mdata->fft_data;
      curfield_band = which_band;
      curfield_type = 'h';
-     maxwell_compute_h_from_H(mdata, H, curfield, which_band - 1, 1);
+     if (mdata->mu_inv == NULL)
+         maxwell_compute_h_from_H(mdata, H, curfield, which_band - 1, 1);
+     else {
+         evectmatrix_resize(&W[0], 1, 0);
+         maxwell_compute_H_from_B(mdata, H, W[0], curfield, which_band-1,0, 1);
+         maxwell_compute_h_from_H(mdata, W[0], curfield, 0, 1);
+         evectmatrix_resize(&W[0], W[0].alloc_p, 0);
+     }
 
-     /* Divide by the cell volume so that the integral of |H|^2
+     /* Divide by the cell volume so that the integral of H*B
         or of D*E is unity.  (From the eigensolver + FFT, they are
         initially normalized to sum to nx*ny*nz.) */
+     {
+	  int i, N;
+	  double scale;
+	  N = mdata->fft_output_size;
 
+	  scale = 1.0 / sqrt(Vol);
+	  for (i = 0; i < 3*N; ++i) {
+	       curfield[i].re *= scale;
+	       curfield[i].im *= scale;
+	  }
+     }
+}
+
+void get_bfield(integer which_band)
+{
+     if (!mdata) {
+	  mpi_one_fprintf(stderr,
+			  "init-params must be called before get-bfield!\n");
+	  return;
+     }
+     if (!kpoint_index) {
+	  mpi_one_fprintf(stderr,
+			  "solve-kpoint must be called before get-bfield!\n");
+	  return;
+     }
+     if (which_band < 1 || which_band > H.p) {
+	  mpi_one_fprintf(stderr,
+			  "must have 1 <= band index <= num_bands (%d)\n",H.p);
+	  return;
+     }
+
+     curfield = (scalar_complex *) mdata->fft_data;
+     curfield_band = which_band;
+     curfield_type = 'b';
+     maxwell_compute_h_from_H(mdata, H, curfield, which_band - 1, 1);
+
+     /* Divide by the cell volume so that the integral of H*B
+        or of D*E is unity.  (From the eigensolver + FFT, they are
+        initially normalized to sum to nx*ny*nz.) */
      {
 	  int i, N;
 	  double scale;
@@ -157,7 +210,7 @@ void get_efield(integer which_band)
 /* Extract the mean epsilon from the effective inverse dielectric tensor,
    which contains two eigenvalues that correspond to the mean epsilon,
    and one which corresponds to the harmonic mean. */
-real mean_epsilon_from_matrix(const symmetric_matrix *eps_inv)
+real mean_medium_from_matrix(const symmetric_matrix *eps_inv)
 {
      real eps_eigs[3];
      maxwell_sym_matrix_eigs(eps_eigs, eps_inv);
@@ -166,155 +219,6 @@ real mean_epsilon_from_matrix(const symmetric_matrix *eps_inv)
      return 2.0 / (eps_eigs[0] + eps_eigs[1]);
 }
 
-/* get the dielectric function, and compute some statistics */
-void get_epsilon(void)
-{
-     int i, N, last_dim, last_dim_stored, nx, nz, local_y_start;
-     real *epsilon;
-     real eps_mean = 0, eps_inv_mean = 0, eps_high = -1e20, eps_low = 1e20;
-     int fill_count = 0;
-
-     if (!mdata) {
-	  mpi_one_fprintf(stderr,
-			  "init-params must be called before get-epsilon!\n");
-	  return;
-     }
-
-     curfield = (scalar_complex *) mdata->fft_data;
-     epsilon = (real *) curfield;
-     curfield_band = 0;
-     curfield_type = 'n';
-
-     /* get epsilon.  Recall that we actually have an inverse
-	dielectric tensor at each point; define an average index by
-	the inverse of the average eigenvalue of the 1/eps tensor.
-	i.e. 3/(trace 1/eps). */
-
-     N = mdata->fft_output_size;
-     last_dim = mdata->last_dim;
-     last_dim_stored =
-	  mdata->last_dim_size / (sizeof(scalar_complex)/sizeof(scalar));
-     nx = mdata->nx; nz = mdata->nz; local_y_start = mdata->local_y_start;
-
-     for (i = 0; i < N; ++i) {
-          epsilon[i] = mean_epsilon_from_matrix(mdata->eps_inv + i);
-	  if (epsilon[i] < eps_low)
-	       eps_low = epsilon[i];
-	  if (epsilon[i] > eps_high)
-	       eps_high = epsilon[i];
-	  eps_mean += epsilon[i];
-	  eps_inv_mean += 1/epsilon[i];
-	  if (epsilon[i] > 1.0001)
-	       ++fill_count;
-#ifndef SCALAR_COMPLEX
-	  /* most points need to be counted twice, by rfftw output symmetry: */
-	  {
-	       int last_index;
-#  ifdef HAVE_MPI
-	       if (nz == 1) /* 2d calculation: 1st dim. is truncated one */
-		    last_index = i / nx + local_y_start;
-	       else
-		    last_index = i % last_dim_stored;
-#  else
-	       last_index = i % last_dim_stored;
-#  endif
-	       if (last_index != 0 && 2*last_index != last_dim) {
-		    eps_mean += epsilon[i];
-		    eps_inv_mean += 1/epsilon[i];
-		    if (epsilon[i] > 1.0001)
-			 ++fill_count;
-	       }
-	  }
-#endif
-     }
-
-     mpi_allreduce_1(&eps_mean, real, SCALAR_MPI_TYPE,
-		     MPI_SUM, MPI_COMM_WORLD);
-     mpi_allreduce_1(&eps_inv_mean, real, SCALAR_MPI_TYPE,
-		     MPI_SUM, MPI_COMM_WORLD);
-     mpi_allreduce_1(&eps_low, real, SCALAR_MPI_TYPE,
-		     MPI_MIN, MPI_COMM_WORLD);
-     mpi_allreduce_1(&eps_high, real, SCALAR_MPI_TYPE,
-		     MPI_MAX, MPI_COMM_WORLD);
-     mpi_allreduce_1(&fill_count, int, MPI_INT,
-                   MPI_SUM, MPI_COMM_WORLD);
-     N = mdata->nx * mdata->ny * mdata->nz;
-     eps_mean /= N;
-     eps_inv_mean = N/eps_inv_mean;
-
-     mpi_one_printf("epsilon: %g-%g, mean %g, harm. mean %g, "
-		    "%g%% > 1, %g%% \"fill\"\n",
-		    eps_low, eps_high, eps_mean, eps_inv_mean,
-		    (100.0 * fill_count) / N, 
-		    eps_high == eps_low ? 100.0 :
-		    100.0 * (eps_mean-eps_low) / (eps_high-eps_low));
-}
-
-/* get the specified component of the dielectric tensor,
-   or the inverse tensor if inv != 0 */
-void get_epsilon_tensor(int c1, int c2, int imag, int inv)
-{
-     int i, N;
-     real *epsilon;
-     int conj = 0, offset = 0;
-
-     curfield_type = '-'; /* only used internally, for now */
-     epsilon = (real *) mdata->fft_data;
-     N = mdata->fft_output_size;
-
-     switch (c1 * 3 + c2) {
-	 case 0:
-	      offset = offsetof(symmetric_matrix, m00);
-	      break;
-	 case 1:
-	      offset = offsetof(symmetric_matrix, m01);
-	      break;
-	 case 2:
-	      offset = offsetof(symmetric_matrix, m02);
-	      break;
-	 case 3:
-	      offset = offsetof(symmetric_matrix, m01); /* = conj(m10) */
-	      conj = imag;
-	      break;
-	 case 4:
-	      offset = offsetof(symmetric_matrix, m11);
-	      break;
-	 case 5:
-	      offset = offsetof(symmetric_matrix, m12);
-	      break;
-	 case 6:
-	      offset = offsetof(symmetric_matrix, m02); /* = conj(m20) */
-	      conj = imag;
-	      break;
-	 case 7:
-	      offset = offsetof(symmetric_matrix, m12); /* = conj(m21) */
-	      conj = imag;
-	      break;
-	 case 8:
-	      offset = offsetof(symmetric_matrix, m22);
-	      break;
-     }
-
-#ifdef WITH_HERMITIAN_EPSILON
-     if (c1 != c2 && imag)
-	  offset += offsetof(scalar_complex, im);
-#endif
-
-     for (i = 0; i < N; ++i) {
-	  if (inv) {
-	       epsilon[i] = 
-		    *((real *) (((char *) &mdata->eps_inv[i]) + offset));
-	  }
-	  else {
-	       symmetric_matrix eps;
-	       maxwell_sym_matrix_invert(&eps, &mdata->eps_inv[i]);
-	       epsilon[i] = *((real *) (((char *) &eps) + offset));
-	  }
-	  if (conj)
-	       epsilon[i] = -epsilon[i];
-     }     
-}
-
 /**************************************************************************/
 
 /* internal function for compute_field_energy, below */
@@ -336,10 +240,12 @@ double compute_field_energy_internal(real comp_sum[6])
 	  real
 	       comp_sqr0,comp_sqr1,comp_sqr2,comp_sqr3,comp_sqr4,comp_sqr5;
 
-	  /* energy is either |curfield|^2 or |curfield|^2 / epsilon,
-	     depending upon whether it is H or D. */
-	  if (curfield_type == 'd') 
+	  /* energy is either |curfield|^2 / mu or |curfield|^2 / epsilon,
+	     depending upon whether it is B or D. */
+	  if (curfield_type == 'd')
 	       assign_symmatrix_vector(field, mdata->eps_inv[i], curfield+3*i);
+	  else if (curfield_type == 'b' && mdata->mu_inv != NULL)
+	       assign_symmatrix_vector(field, mdata->mu_inv[i], curfield+3*i);
 	  else {
 	       field[0] =   curfield[3*i];
 	       field[1] = curfield[3*i+1];
@@ -358,7 +264,7 @@ double compute_field_energy_internal(real comp_sum[6])
 	     since energy_density[i] is guaranteed to come at or before
 	     curfield[i] (which we are now done with). */
 
-	  energy_sum += energy_density[i] = 
+	  energy_sum += energy_density[i] =
 	       comp_sqr0+comp_sqr1+comp_sqr2+comp_sqr3+comp_sqr4+comp_sqr5;
 #ifndef SCALAR_COMPLEX
 	  /* most points need to be counted twice, by rfftw output symmetry: */
@@ -386,9 +292,9 @@ double compute_field_energy_internal(real comp_sum[6])
      }
 
      mpi_allreduce_1(&energy_sum, real, SCALAR_MPI_TYPE,
-		     MPI_SUM, MPI_COMM_WORLD);
+		     MPI_SUM, mpb_comm);
      mpi_allreduce(comp_sum2, comp_sum, 6, real, SCALAR_MPI_TYPE,
-                   MPI_SUM, MPI_COMM_WORLD);
+                   MPI_SUM, mpb_comm);
 
      /* remember that we now have energy density; denoted by capital D/H */
      curfield_type = toupper(curfield_type);
@@ -406,10 +312,14 @@ number_list compute_field_energy(void)
      real energy_sum, comp_sum[6];
      number_list retval = { 0, 0 };
 
-     if (!curfield || !strchr("dh", curfield_type)) {
+     if (!curfield || !strchr("dhb", curfield_type)) {
 	  mpi_one_fprintf(stderr, "The D or H field must be loaded first.\n");
 	  return retval;
      }
+     else if (curfield_type == 'h' && mdata->mu_inv != NULL) {
+	  mpi_one_fprintf(stderr, "B, not H, must be loaded if we have mu.\n");
+	  return retval;
+     }
 
      energy_sum = compute_field_energy_internal(comp_sum);
 
@@ -443,7 +353,7 @@ void compute_field_squared(void)
 {
      real comp_sum[6]; /* unused */
 
-     if (!curfield || !strchr("dhecv", curfield_type)) {
+     if (!curfield || !strchr("dhbecv", curfield_type)) {
           mpi_one_fprintf(stderr, "A vector field must be loaded first.\n");
      }
 
@@ -464,7 +374,7 @@ void compute_field_divergence(void)
      scalar *field2 = mdata->fft_data == mdata->fft_data2 ? field : (field == mdata->fft_data ? mdata->fft_data2 : mdata->fft_data);
      real scale;
 
-     if (!curfield || !strchr("dhec", curfield_type)) {
+     if (!curfield || !strchr("dhbec", curfield_type)) {
           mpi_one_fprintf(stderr, "A Bloch-periodic field must be loaded.\n");
           return;
      }
@@ -508,7 +418,7 @@ void compute_field_divergence(void)
 
 /**************************************************************************/
 
-/* Fix the phase of the current field (e/h/d) to a canonical value.
+/* Fix the phase of the current field (e/h/b/d) to a canonical value.
    Also changes the phase of the corresponding eigenvector by the
    same amount, so that future calculations will have a consistent
    phase.
@@ -534,7 +444,7 @@ void fix_field_phase(void)
      double theta;
      scalar phase;
 
-     if (!curfield || !strchr("dhecv", curfield_type)) {
+     if (!curfield || !strchr("dhbecv", curfield_type)) {
           mpi_one_fprintf(stderr, "The D/H/E field must be loaded first.\n");
           return;
      }
@@ -551,7 +461,7 @@ void fix_field_phase(void)
 	  sq_sum2[1] += 2*a*b;
      }
      mpi_allreduce(sq_sum2, sq_sum, 2, real, SCALAR_MPI_TYPE,
-                   MPI_SUM, MPI_COMM_WORLD);
+                   MPI_SUM, mpb_comm);
      /* compute the phase = exp(i*theta) maximizing the real part of
 	the sum of the squares.  i.e., maximize:
 	    cos(2*theta)*sq_sum[0] - sin(2*theta)*sq_sum[1] */
@@ -566,7 +476,7 @@ void fix_field_phase(void)
 	maximum |real part| of the jmax component (after multiplying
 	by phase), and then finding the last spatial index at which
 	|real part| is at least half of this value.  The sign is then
-	chosen to make the real part positive at that point. 
+	chosen to make the real part positive at that point.
 
         (Note that we can't just make the point of maximum |real part|
          positive, as that would be ambiguous in the common case of an
@@ -586,7 +496,7 @@ void fix_field_phase(void)
 	       maxabs = r;
      }
      mpi_allreduce_1(&maxabs, real, SCALAR_MPI_TYPE,
-		     MPI_MAX, MPI_COMM_WORLD);
+		     MPI_MAX, mpb_comm);
      for (i = N - 1; i >= 0; --i) {
 #ifdef SCALAR_COMPLEX
 	  real r = curfield[i].re * phase.re - curfield[i].im * phase.im;
@@ -602,12 +512,12 @@ void fix_field_phase(void)
      if (i >= 0)  /* convert index to global index in distributed array: */
 	  maxabs_index += mdata->local_y_start * mdata->nx * mdata->nz;
      {
-	  /* compute maximum index and corresponding sign over all the 
+	  /* compute maximum index and corresponding sign over all the
 	     processors, using the MPI_MAXLOC reduction operation: */
 	  struct twoint_struct {int i; int s;} x;
 	  x.i = maxabs_index; x.s = maxabs_sign;
 	  mpi_allreduce_1(&x, struct twoint_struct, MPI_2INT,
-			  MPI_MAXLOC, MPI_COMM_WORLD);
+			  MPI_MAXLOC, mpb_comm);
 	  maxabs_index = x.i; maxabs_sign = x.s;
      }
      ASSIGN_SCALAR(phase,
@@ -627,7 +537,7 @@ void fix_field_phase(void)
 	  curfield[i].im = a*SCALAR_IM(phase) + b*SCALAR_RE(phase);
      }
      for (i = 0; i < H.n; ++i) {
-          ASSIGN_MULT(H.data[i*H.p + curfield_band - 1], 
+          ASSIGN_MULT(H.data[i*H.p + curfield_band - 1],
 		      H.data[i*H.p + curfield_band - 1], phase);
      }
 }
@@ -709,7 +619,7 @@ static real interp_val(vector3 p, int nx, int ny, int nz, int last_dim_size,
 #undef D
 }
 
-static scalar_complex interp_cval(vector3 p, 
+static scalar_complex interp_cval(vector3 p,
 				  int nx, int ny, int nz, int last_dim_size,
 				  real *data, int stride)
 {
@@ -746,7 +656,7 @@ number get_epsilon_point(vector3 p)
 {
      symmetric_matrix eps_inv;
      eps_inv = interp_eps_inv(p);
-     return mean_epsilon_from_matrix(&eps_inv);
+     return mean_medium_from_matrix(&eps_inv);
 }
 
 cmatrix3x3 get_epsilon_inverse_tensor_point(vector3 p)
@@ -769,7 +679,7 @@ cmatrix3x3 get_epsilon_inverse_tensor_point(vector3 p)
 
 number get_energy_point(vector3 p)
 {
-     CHECK(curfield && strchr("DHR", curfield_type),
+     CHECK(curfield && strchr("DHBR", curfield_type),
 	   "compute-field-energy must be called before get-energy-point");
      return f_interp_val(p, mdata, (real *) curfield, 1, 0);
 }
@@ -779,7 +689,7 @@ cvector3 get_bloch_field_point(vector3 p)
      scalar_complex field[3];
      cvector3 F;
 
-     CHECK(curfield && strchr("dhecv", curfield_type),
+     CHECK(curfield && strchr("dhbecv", curfield_type),
 	   "field must be must be loaded before get-*field*-point");
      field[0] = f_interp_cval(p, mdata, &curfield[0].re, 6);
      field[1] = f_interp_cval(p, mdata, &curfield[1].re, 6);
@@ -795,14 +705,14 @@ cvector3 get_field_point(vector3 p)
      scalar_complex field[3], phase;
      cvector3 F;
 
-     CHECK(curfield && strchr("dhecv", curfield_type),
+     CHECK(curfield && strchr("dhbecv", curfield_type),
 	   "field must be must be loaded before get-*field*-point");
      field[0] = f_interp_cval(p, mdata, &curfield[0].re, 6);
      field[1] = f_interp_cval(p, mdata, &curfield[1].re, 6);
      field[2] = f_interp_cval(p, mdata, &curfield[2].re, 6);
 
      if (curfield_type != 'v') {
-	  double phase_phi = TWOPI * 
+	  double phase_phi = TWOPI *
 	       (cur_kvector.x * (p.x/geometry_lattice.size.x) +
 		cur_kvector.y * (p.y/geometry_lattice.size.y) +
 		cur_kvector.z * (p.z/geometry_lattice.size.z));
@@ -822,7 +732,7 @@ cnumber get_bloch_cscalar_point(vector3 p)
 {
      CHECK(curfield && strchr("C", curfield_type),
 	   "cscalar must be must be loaded before get-*cscalar*-point");
-     
+
      return cscalar2cnumber(f_interp_cval(p, mdata, &curfield[0].re, 2));
 }
 
@@ -832,12 +742,12 @@ cnumber get_cscalar_point(vector3 p)
 
      CHECK(curfield && strchr("C", curfield_type),
 	   "cscalar must be must be loaded before get-*cscalar*-point");
-     
+
      s = f_interp_cval(p, mdata, &curfield[0].re, 2);
 
      if (curfield_type == 'C') {
 	  scalar_complex phase;
-	  double phase_phi = TWOPI * 
+	  double phase_phi = TWOPI *
 	       (cur_kvector.x * (p.x/geometry_lattice.size.x) +
 		cur_kvector.y * (p.y/geometry_lattice.size.y) +
 		cur_kvector.z * (p.z/geometry_lattice.size.z));
@@ -851,7 +761,7 @@ cnumber get_cscalar_point(vector3 p)
 number rscalar_field_get_point(SCM fo, vector3 p)
 {
      field_smob *f = assert_field_smob(fo);
-     CHECK(f->type == RSCALAR_FIELD_SMOB, 
+     CHECK(f->type == RSCALAR_FIELD_SMOB,
 	   "invalid argument to rscalar-field-get-point");
      return f_interp_val(p, f, f->f.rs, 1, 0);
 }
@@ -861,7 +771,7 @@ cvector3 cvector_field_get_point_bloch(SCM fo, vector3 p)
      scalar_complex field[3];
      cvector3 F;
      field_smob *f = assert_field_smob(fo);
-     CHECK(f->type == CVECTOR_FIELD_SMOB, 
+     CHECK(f->type == CVECTOR_FIELD_SMOB,
 	   "invalid argument to cvector-field-get-point");
      field[0] = f_interp_cval(p, f, &f->f.cv[0].re, 6);
      field[1] = f_interp_cval(p, f, &f->f.cv[1].re, 6);
@@ -877,16 +787,16 @@ cvector3 cvector_field_get_point(SCM fo, vector3 p)
      scalar_complex field[3];
      cvector3 F;
      field_smob *f = assert_field_smob(fo);
-     CHECK(f->type == CVECTOR_FIELD_SMOB, 
+     CHECK(f->type == CVECTOR_FIELD_SMOB,
 	   "invalid argument to cvector-field-get-point");
 
      field[0] = f_interp_cval(p, f, &f->f.cv[0].re, 6);
      field[1] = f_interp_cval(p, f, &f->f.cv[1].re, 6);
      field[2] = f_interp_cval(p, f, &f->f.cv[2].re, 6);
-     
+
      if (f->type_char != 'v') { /* v fields have no kvector */
 	  scalar_complex phase;
-	  double phase_phi = TWOPI * 
+	  double phase_phi = TWOPI *
 	       (cur_kvector.x * (p.x/geometry_lattice.size.x) +
 		cur_kvector.y * (p.y/geometry_lattice.size.y) +
 		cur_kvector.z * (p.z/geometry_lattice.size.z));
@@ -905,7 +815,7 @@ cvector3 cvector_field_get_point(SCM fo, vector3 p)
 cnumber cscalar_field_get_point_bloch(SCM fo, vector3 p)
 {
      field_smob *f = assert_field_smob(fo);
-     CHECK(f->type == CSCALAR_FIELD_SMOB, 
+     CHECK(f->type == CSCALAR_FIELD_SMOB,
 	   "invalid argument to cscalar-field-get-point-bloch");
      return cscalar2cnumber(f_interp_cval(p, f, &f->f.cv[0].re, 2));
 }
@@ -914,14 +824,14 @@ cnumber cscalar_field_get_point(SCM fo, vector3 p)
 {
      scalar_complex s;
      field_smob *f = assert_field_smob(fo);
-     CHECK(f->type == CSCALAR_FIELD_SMOB, 
+     CHECK(f->type == CSCALAR_FIELD_SMOB,
 	   "invalid argument to cscalar-field-get-point");
 
      s = f_interp_cval(p, f, &f->f.cv[0].re, 2);
-     
+
      if (f->type_char == 'C') { /* have kvector */
 	  scalar_complex phase;
-	  double phase_phi = TWOPI * 
+	  double phase_phi = TWOPI *
 	       (cur_kvector.x * (p.x/geometry_lattice.size.x) +
 		cur_kvector.y * (p.y/geometry_lattice.size.y) +
 		cur_kvector.z * (p.z/geometry_lattice.size.z));
@@ -942,7 +852,7 @@ number compute_energy_in_dielectric(number eps_low, number eps_high)
      real *energy = (real *) curfield;
      real epsilon, energy_sum = 0.0;
 
-     if (!curfield || !strchr("DHR", curfield_type)) {
+     if (!curfield || !strchr("DHBR", curfield_type)) {
           mpi_one_fprintf(stderr, "The D or H energy density must be loaded first.\n");
           return 0.0;
      }
@@ -954,7 +864,7 @@ number compute_energy_in_dielectric(number eps_low, number eps_high)
      nx = mdata->nx; nz = mdata->nz; local_y_start = mdata->local_y_start;
 
      for (i = 0; i < N; ++i) {
-	  epsilon = mean_epsilon_from_matrix(mdata->eps_inv +i);
+	  epsilon = mean_medium_from_matrix(mdata->eps_inv +i);
 	  if (epsilon >= eps_low && epsilon <= eps_high) {
 	       energy_sum += energy[i];
 #ifndef SCALAR_COMPLEX
@@ -976,7 +886,7 @@ number compute_energy_in_dielectric(number eps_low, number eps_high)
 	  }
      }
      mpi_allreduce_1(&energy_sum, real, SCALAR_MPI_TYPE,
-		     MPI_SUM, MPI_COMM_WORLD);
+		     MPI_SUM, mpb_comm);
      energy_sum *= Vol / H.N;
      return energy_sum;
 }
@@ -992,7 +902,7 @@ static char *fix_fname(const char *fname, const char *prefix,
 {
      char *s;
      CHK_MALLOC(s, char,
-		(fname ? strlen(fname) : 0) + 
+		(fname ? strlen(fname) : 0) +
 		(prefix ? strlen(prefix) : 0) + 20);
      strcpy(s, prefix ? prefix : "");
      strcat(s, fname ? fname : "");
@@ -1018,10 +928,10 @@ static void output_scalarfield(real *vals,
 {
      matrixio_id data_id = {-1, 1};
 
-     fieldio_write_real_vals(vals, 3, dims, 
-			     local_dims, start, file_id, 0, 
+     fieldio_write_real_vals(vals, 3, dims,
+			     local_dims, start, file_id, 0,
 			     dataname, &data_id);
-     
+
 #ifndef SCALAR_COMPLEX
      {
 	  int start_new[3], local_dims_new[3];
@@ -1044,17 +954,17 @@ static void output_scalarfield(real *vals,
 		  then write the start_new[0] slab. */
 	       fieldio_write_real_vals(vals +
 				       local_dims_new[1] * local_dims_new[2],
-				       3, dims, local_dims_new, start_new, 
+				       3, dims, local_dims_new, start_new,
 				       file_id, 1, dataname, &data_id);
 	       local_dims_new[0] = 1;
 	       start_new[0] = 0;
-	       fieldio_write_real_vals(vals, 3, dims, 
+	       fieldio_write_real_vals(vals, 3, dims,
 				       local_dims_new, start_new,
 				       file_id, 1, dataname, &data_id);
 	  }
 	  else {
-	       fieldio_write_real_vals(vals, 3, dims, 
-				       local_dims_new, start_new, 
+	       fieldio_write_real_vals(vals, 3, dims,
+				       local_dims_new, start_new,
 				       file_id, 1, dataname, &data_id);
 	  }
      }
@@ -1066,7 +976,7 @@ static void output_scalarfield(real *vals,
 
 /* given the field in curfield, store it to HDF (or whatever) using
    the matrixio (fieldio) routines.  Allow the component to be specified
-   (which_component 0/1/2 = x/y/z, -1 = all) for vector fields. 
+   (which_component 0/1/2 = x/y/z, -1 = all) for vector fields.
    Also allow the user to specify a prefix string for the filename. */
 void output_field_to_file(integer which_component, string filename_prefix)
 {
@@ -1084,11 +994,11 @@ void output_field_to_file(integer which_component, string filename_prefix)
      int write_start0_special = 0;
 
      if (!curfield) {
-	  mpi_one_fprintf(stderr, 
+	  mpi_one_fprintf(stderr,
 		  "fields, energy dens., or epsilon must be loaded first.\n");
 	  return;
      }
-     
+
 #ifdef HAVE_MPI
      /* The first two dimensions (x and y) of the position-space fields
 	are transposed when we use MPI, so we need to transpose everything. */
@@ -1167,8 +1077,8 @@ void output_field_to_file(integer which_component, string filename_prefix)
 
      if (strchr("Rv", curfield_type)) /* generic scalar/vector field */
 	  output_k[0] = output_k[1] = output_k[2] = 0.0; /* don't know k */
-     
-     if (strchr("dhecv", curfield_type)) { /* outputting vector field */
+
+     if (strchr("dhbecv", curfield_type)) { /* outputting vector field */
 	  matrixio_id data_id[6] = {{-1,1},{-1,1},{-1,1},{-1,1},{-1,1},{-1,1}};
 	  int i;
 
@@ -1180,7 +1090,7 @@ void output_field_to_file(integer which_component, string filename_prefix)
 	       strcat(fname, comp_str);
 	  }
 	  sprintf(description, "%c field, kpoint %d, band %d, freq=%g",
-		  curfield_type, kpoint_index, curfield_band, 
+		  curfield_type, kpoint_index, curfield_band,
 		  freqs.items[curfield_band - 1]);
 	  fname2 = fix_fname(fname, filename_prefix, mdata, 1);
 	  mpi_one_printf("Outputting fields to %s...\n", fname2);
@@ -1202,10 +1112,10 @@ void output_field_to_file(integer which_component, string filename_prefix)
 	       /* The conjugated array half may be discontiguous.
 		  First, write the part not containing start[0], and
 		  then write the start[0] slab. */
-	       fieldio_write_complex_field(curfield + 
+	       fieldio_write_complex_field(curfield +
 					   3 * local_dims[1] * local_dims[2],
 					   3, dims, local_dims, start,
-					   which_component, 3, NULL, 
+					   which_component, 3, NULL,
 					   file_id, 1, data_id);
 	       local_dims[0] = 1;
 	       start[0] = 0;
@@ -1233,7 +1143,7 @@ void output_field_to_file(integer which_component, string filename_prefix)
 	  sprintf(fname, "%c.k%02d.b%02d",
 		  curfield_type, kpoint_index, curfield_band);
 	  sprintf(description, "%c field, kpoint %d, band %d, freq=%g",
-		  curfield_type, kpoint_index, curfield_band, 
+		  curfield_type, kpoint_index, curfield_band,
 		  freqs.items[curfield_band - 1]);
 	  fname2 = fix_fname(fname, filename_prefix, mdata, 1);
 	  mpi_one_printf("Outputting complex scalar field to %s...\n", fname2);
@@ -1255,10 +1165,10 @@ void output_field_to_file(integer which_component, string filename_prefix)
 	       /* The conjugated array half may be discontiguous.
 		  First, write the part not containing start[0], and
 		  then write the start[0] slab. */
-	       fieldio_write_complex_field(curfield + 
+	       fieldio_write_complex_field(curfield +
 					   local_dims[1] * local_dims[2],
 					   3, dims, local_dims, start,
-					   which_component, 1, NULL, 
+					   which_component, 1, NULL,
 					   file_id, 1, data_id);
 	       local_dims[0] = 1;
 	       start[0] = 0;
@@ -1279,27 +1189,31 @@ void output_field_to_file(integer which_component, string filename_prefix)
 	  matrixio_write_data_attr(file_id, "Bloch wavevector",
 				   output_k, 1, attr_dims);
      }
-     else if (strchr("DHnR", curfield_type)) { /* scalar field */
+     else if (strchr("DHBnmR", curfield_type)) { /* scalar field */
 	  if (curfield_type == 'n') {
 	       sprintf(fname, "epsilon");
 	       sprintf(description, "dielectric function, epsilon");
 	  }
+	  else if (curfield_type == 'm') {
+	       sprintf(fname, "mu");
+	       sprintf(description, "permeability mu");
+	  }
 	  else {
 	       sprintf(fname, "%cpwr.k%02d.b%02d",
 		       tolower(curfield_type), kpoint_index, curfield_band);
 	       sprintf(description,
 		       "%c field energy density, kpoint %d, band %d, freq=%g",
-		       curfield_type, kpoint_index, curfield_band, 
+		       curfield_type, kpoint_index, curfield_band,
 		       freqs.items[curfield_band - 1]);
 	  }
-	  fname2 = fix_fname(fname, filename_prefix, mdata, 
+	  fname2 = fix_fname(fname, filename_prefix, mdata,
 			     /* no parity suffix for epsilon: */
-			     curfield_type != 'n');
+			     curfield_type != 'n' && curfield_type != 'm');
 	  mpi_one_printf("Outputting %s...\n", fname2);
 	  file_id = matrixio_create(fname2);
 	  free(fname2);
 
-	  output_scalarfield((real *) curfield, dims, 
+	  output_scalarfield((real *) curfield, dims,
 			     local_dims, start, file_id, "data",
 			     last_dim_index, last_dim_start, last_dim_size,
 			     first_dim_start, first_dim_size,
@@ -1378,7 +1292,7 @@ number compute_energy_in_object_list(geometric_object_list objects)
      real *energy = (real *) curfield;
      real energy_sum = 0;
 
-     if (!curfield || !strchr("DHR", curfield_type)) {
+     if (!curfield || !strchr("DHBR", curfield_type)) {
           mpi_one_fprintf(stderr, "The D or H energy density must be loaded first.\n");
           return 0.0;
      }
@@ -1399,94 +1313,16 @@ number compute_energy_in_object_list(geometric_object_list objects)
      c2 = n2 <= 1 ? 0 : geometry_lattice.size.y * 0.5;
      c3 = n3 <= 1 ? 0 : geometry_lattice.size.z * 0.5;
 
-     /* Here we have different loops over the coordinates, depending
-	upon whether we are using complex or real and serial or
-        parallel transforms.  Each loop must define, in its body,
-        variables (i2,j2,k2) describing the coordinate of the current
-        point, and "index" describing the corresponding index in 
-	the curfield array.
-
-        This was all stolen from maxwell_eps.c...it would be better
-        if we didn't have to cut and paste, sigh. */
-
-#ifdef SCALAR_COMPLEX
-
-#  ifndef HAVE_MPI
-     
-     for (i = 0; i < n1; ++i)
-	  for (j = 0; j < n2; ++j)
-	       for (k = 0; k < n3; ++k)
-     {
-	  int i2 = i, j2 = j, k2 = k;
-	  int index = ((i * n2 + j) * n3 + k);
-
-#  else /* HAVE_MPI */
-
-     local_n2 = mdata->local_ny;
-     local_y_start = mdata->local_y_start;
-
-     /* first two dimensions are transposed in MPI output: */
-     for (j = 0; j < local_n2; ++j)
-          for (i = 0; i < n1; ++i)
-	       for (k = 0; k < n3; ++k)
-     {
-	  int i2 = i, j2 = j + local_y_start, k2 = k;
-	  int index = ((j * n1 + i) * n3 + k);
-
-#  endif /* HAVE_MPI */
-
-#else /* not SCALAR_COMPLEX */
-
-#  ifndef HAVE_MPI
-
-     for (i = 0; i < n_other; ++i)
-	  for (j = 0; j < n_last; ++j)
-     {
-	  int index = i * n_last + j;
-	  int i2, j2, k2;
-	  switch (rank) {
-	      case 2: i2 = i; j2 = j; k2 = 0; break;
-	      case 3: i2 = i / n2; j2 = i % n2; k2 = j; break;
-	      default: i2 = j; j2 = k2 = 0;  break;
-	  }
-
-#  else /* HAVE_MPI */
-
-     local_n2 = mdata->local_ny;
-     local_y_start = mdata->local_y_start;
-
-     /* For a real->complex transform, the last dimension is cut in
-	half.  For a 2d transform, this is taken into account in local_ny
-	already, but for a 3d transform we must compute the new n3: */
-     if (n3 > 1)
-	  local_n3 = mdata->last_dim_size / 2;
-     else
-	  local_n3 = 1;
-     
-     /* first two dimensions are transposed in MPI output: */
-     for (j = 0; j < local_n2; ++j)
-          for (i = 0; i < n1; ++i)
-	       for (k = 0; k < local_n3; ++k)
-     {
-#         define i2 i
-	  int j2 = j + local_y_start;
-#         define k2 k
-	  int index = ((j * n1 + i) * local_n3 + k);
-
-#  endif /* HAVE_MPI */
-
-#endif /* not SCALAR_COMPLEX */
-
-	  {
+     LOOP_XYZ(mdata) {
 	       vector3 p;
 	       int n;
-	       p.x = i2 * s1 - c1; p.y = j2 * s2 - c2; p.z = k2 * s3 - c3;
+	       p.x = i1 * s1 - c1; p.y = i2 * s2 - c2; p.z = i3 * s3 - c3;
 	       for (n = objects.num_items - 1; n >= 0; --n)
 		    if (point_in_periodic_fixed_objectp(p, objects.items[n])) {
 			 if (objects.items[n].material.which_subclass
 			     == MATERIAL_TYPE_SELF)
 			      break; /* treat as a "nothing" object */
-			 energy_sum += energy[index];
+			 energy_sum += energy[xyz_index];
 #ifndef SCALAR_COMPLEX
 			 {
 			      int last_index;
@@ -1499,16 +1335,15 @@ number compute_energy_in_object_list(geometric_object_list objects)
 			      last_index = j;
 #  endif
 			      if (last_index != 0 && 2*last_index != last_dim)
-				   energy_sum += energy[index];
+				   energy_sum += energy[xyz_index];
 			 }
 #endif
 			 break;
 		    }
-	  }
-     }
+	}}}
 
      mpi_allreduce_1(&energy_sum, real, SCALAR_MPI_TYPE,
-		     MPI_SUM, MPI_COMM_WORLD);
+		     MPI_SUM, mpb_comm);
      energy_sum *= Vol / H.N;
      return energy_sum;
 }
@@ -1528,14 +1363,14 @@ cnumber compute_field_integral(function f)
      cnumber integral = {0,0};
      vector3 kvector = {0,0,0};
 
-     if (!curfield || !strchr("dheDHRcv", curfield_type)) {
+     if (!curfield || !strchr("dhbeDHBRcv", curfield_type)) {
           mpi_one_fprintf(stderr, "The D or H energy/field must be loaded first.\n");
           return integral;
      }
      if (curfield_type != 'v')
 	  kvector = cur_kvector;
 
-     integrate_energy = strchr("DHR", curfield_type) != NULL;
+     integrate_energy = strchr("DHBR", curfield_type) != NULL;
 
      n1 = mdata->nx; n2 = mdata->ny; n3 = mdata->nz;
      n_other = mdata->other_dims;
@@ -1550,98 +1385,20 @@ cnumber compute_field_integral(function f)
      c2 = n2 <= 1 ? 0 : geometry_lattice.size.y * 0.5;
      c3 = n3 <= 1 ? 0 : geometry_lattice.size.z * 0.5;
 
-     /* Here we have different loops over the coordinates, depending
-	upon whether we are using complex or real and serial or
-        parallel transforms.  Each loop must define, in its body,
-        variables (i2,j2,k2) describing the coordinate of the current
-        point, and "index" describing the corresponding index in 
-	the curfield array.
-
-        This was all stolen from maxwell_eps.c...it would be better
-        if we didn't have to cut and paste, sigh. */
-
-#ifdef SCALAR_COMPLEX
-
-#  ifndef HAVE_MPI
-     
-     for (i = 0; i < n1; ++i)
-	  for (j = 0; j < n2; ++j)
-	       for (k = 0; k < n3; ++k)
-     {
-	  int i2 = i, j2 = j, k2 = k;
-	  int index = ((i * n2 + j) * n3 + k);
-
-#  else /* HAVE_MPI */
-
-     local_n2 = mdata->local_ny;
-     local_y_start = mdata->local_y_start;
-
-     /* first two dimensions are transposed in MPI output: */
-     for (j = 0; j < local_n2; ++j)
-          for (i = 0; i < n1; ++i)
-	       for (k = 0; k < n3; ++k)
-     {
-	  int i2 = i, j2 = j + local_y_start, k2 = k;
-	  int index = ((j * n1 + i) * n3 + k);
-
-#  endif /* HAVE_MPI */
-
-#else /* not SCALAR_COMPLEX */
-
-#  ifndef HAVE_MPI
-
-     for (i = 0; i < n_other; ++i)
-	  for (j = 0; j < n_last; ++j)
-     {
-	  int index = i * n_last + j;
-	  int i2, j2, k2;
-	  switch (rank) {
-	      case 2: i2 = i; j2 = j; k2 = 0; break;
-	      case 3: i2 = i / n2; j2 = i % n2; k2 = j; break;
-	      default: i2 = j; j2 = k2 = 0;  break;
-	  }
-
-#  else /* HAVE_MPI */
-
-     local_n2 = mdata->local_ny;
-     local_y_start = mdata->local_y_start;
-
-     /* For a real->complex transform, the last dimension is cut in
-	half.  For a 2d transform, this is taken into account in local_ny
-	already, but for a 3d transform we must compute the new n3: */
-     if (n3 > 1)
-	  local_n3 = mdata->last_dim_size / 2;
-     else
-	  local_n3 = 1;
-     
-     /* first two dimensions are transposed in MPI output: */
-     for (j = 0; j < local_n2; ++j)
-          for (i = 0; i < n1; ++i)
-	       for (k = 0; k < local_n3; ++k)
-     {
-#         define i2 i
-	  int j2 = j + local_y_start;
-#         define k2 k
-	  int index = ((j * n1 + i) * local_n3 + k);
-
-#  endif /* HAVE_MPI */
-
-#endif /* not SCALAR_COMPLEX */
-
-	  {
+     LOOP_XYZ(mdata) {
 	       real epsilon;
 	       vector3 p;
 
-	       epsilon = mean_epsilon_from_matrix(mdata->eps_inv + index);
-	       
-	       p.x = i2 * s1 - c1; p.y = j2 * s2 - c2; p.z = k2 * s3 - c3;
+	       epsilon = mean_medium_from_matrix(mdata->eps_inv + xyz_index);
+
+	       p.x = i1 * s1 - c1; p.y = i2 * s2 - c2; p.z = i3 * s3 - c3;
 	       if (integrate_energy) {
 		    integral.re +=
 			 ctl_convert_number_to_c(
 			      gh_call3(f,
-				     ctl_convert_number_to_scm(energy[index]),
+				     ctl_convert_number_to_scm(energy[xyz_index]),
 				       ctl_convert_number_to_scm(epsilon),
-				       ctl_convert_vector3_to_scm(p)));	  
+				       ctl_convert_vector3_to_scm(p)));
 	       }
 	       else {
 		    cvector3 F;
@@ -1649,17 +1406,17 @@ cnumber compute_field_integral(function f)
 		    scalar_complex phase;
 		    cnumber integrand;
 
-		    phase_phi = TWOPI * 
+		    phase_phi = TWOPI *
 			 (kvector.x * (p.x/geometry_lattice.size.x) +
 			  kvector.y * (p.y/geometry_lattice.size.y) +
 			  kvector.z * (p.z/geometry_lattice.size.z));
 		    CASSIGN_SCALAR(phase, cos(phase_phi), sin(phase_phi));
-		    CASSIGN_MULT_RE(F.x.re, curfield[3*index+0], phase);
-		    CASSIGN_MULT_IM(F.x.im, curfield[3*index+0], phase);
-		    CASSIGN_MULT_RE(F.y.re, curfield[3*index+1], phase);
-		    CASSIGN_MULT_IM(F.y.im, curfield[3*index+1], phase);
-		    CASSIGN_MULT_RE(F.z.re, curfield[3*index+2], phase);
-		    CASSIGN_MULT_IM(F.z.im, curfield[3*index+2], phase);
+		    CASSIGN_MULT_RE(F.x.re, curfield[3*xyz_index+0], phase);
+		    CASSIGN_MULT_IM(F.x.im, curfield[3*xyz_index+0], phase);
+		    CASSIGN_MULT_RE(F.y.re, curfield[3*xyz_index+1], phase);
+		    CASSIGN_MULT_IM(F.y.im, curfield[3*xyz_index+1], phase);
+		    CASSIGN_MULT_RE(F.z.re, curfield[3*xyz_index+2], phase);
+		    CASSIGN_MULT_IM(F.z.im, curfield[3*xyz_index+2], phase);
 		    integrand =
 			 ctl_convert_cnumber_to_c(
                               gh_call3(f,
@@ -1681,20 +1438,20 @@ cnumber compute_field_integral(function f)
 #  else
 		    last_index = j;
 #  endif
-		    
+
 		    if (last_index != 0 && 2*last_index != last_dim) {
-			 int i2c, j2c, k2c;
-			 i2c = i2 ? (n1 - i2) : 0;
-			 j2c = j2 ? (n2 - j2) : 0;
-			 k2c = k2 ? (n3 - k2) : 0;
-			 p.x = i2c * s1 - c1; 
-			 p.y = j2c * s2 - c2; 
-			 p.z = k2c * s3 - c3;
+			 int i1c, i2c, i3c;
+			 i1c = i1 ? (n1 - i1) : 0;
+			 i2c = i2 ? (n2 - i2) : 0;
+			 i3c = i3 ? (n3 - i3) : 0;
+			 p.x = i1c * s1 - c1;
+			 p.y = i2c * s2 - c2;
+			 p.z = i3c * s3 - c3;
 			 if (integrate_energy)
-			      integral.re += 
+			      integral.re +=
 				   ctl_convert_number_to_c(
 					gh_call3(f,
-				      ctl_convert_number_to_scm(energy[index]),
+				      ctl_convert_number_to_scm(energy[xyz_index]),
 				      ctl_convert_number_to_scm(epsilon),
 				      ctl_convert_vector3_to_scm(p)));
 			 else {
@@ -1702,20 +1459,20 @@ cnumber compute_field_integral(function f)
 			      double phase_phi;
 			      scalar_complex phase, Fx, Fy, Fz;
 			      cnumber integrand;
-			      
-			      Fx = curfield[3*index+0];
-			      Fy = curfield[3*index+1];
-			      Fz = curfield[3*index+2];
+
+			      Fx = curfield[3*xyz_index+0];
+			      Fy = curfield[3*xyz_index+1];
+			      Fz = curfield[3*xyz_index+2];
 			      Fx.im= -Fx.im; Fy.im= -Fy.im; Fz.im= -Fz.im;
 
-			      phase_phi = TWOPI * 
-				   (kvector.x 
+			      phase_phi = TWOPI *
+				   (kvector.x
 				    * (p.x/geometry_lattice.size.x) +
-				    kvector.y 
+				    kvector.y
 				    * (p.y/geometry_lattice.size.y) +
-				    kvector.z 
+				    kvector.z
 				    * (p.z/geometry_lattice.size.z));
-			      CASSIGN_SCALAR(phase, 
+			      CASSIGN_SCALAR(phase,
 					     cos(phase_phi), sin(phase_phi));
 			      CASSIGN_MULT_RE(F.x.re, Fx, phase);
 			      CASSIGN_MULT_IM(F.x.im, Fx, phase);
@@ -1736,22 +1493,21 @@ cnumber compute_field_integral(function f)
 		    }
 	       }
 #endif
-	  }
-     }
+	}}}
 
      integral.re *= Vol / H.N;
      integral.im *= Vol / H.N;
      {
 	  cnumber integral_sum;
-	  mpi_allreduce(&integral, &integral_sum, 2, number, 
-			MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);
+	  mpi_allreduce(&integral, &integral_sum, 2, number,
+			MPI_DOUBLE, MPI_SUM, mpb_comm);
 	  return integral_sum;
      }
 }
 
 number compute_energy_integral(function f)
 {
-     if (!curfield || !strchr("DHR", curfield_type)) {
+     if (!curfield || !strchr("DHBR", curfield_type)) {
           mpi_one_fprintf(stderr, "The D or H energy density must be loaded first.\n");
           return 0.0;
      }
