@@ -111,8 +111,22 @@ extern void F(heev,HEEV) (char *, char *, int *, scalar *, int *, real *,
 			  scalar *, int *, real *, int *);
 extern void FR(syev,SYEV) (char *, char *, int *, real *, int *, real *,
 			   real *, int *, int *);
+extern void F(hegv,HEGV) (int *, char *, char *, int *, scalar *, int *, scalar *, int *, real *, scalar *, int *, real *, int *);
+extern void FR(sygv,SYGV) (int *, char *, char *, int *, real *, int *, real *, int *, real *, real *, int *, int *);
 
-#ifdef __cplusplus
+#ifdef SCALAR_COMPLEX
+extern void F(geev,GEEV) (char *jobvl, char *jobvr, int *n,
+                          scalar *A, int *lda, scalar *w,
+                          scalar *VL, int *ldvl, scalar *VR, int *ldvr,
+                          scalar *work, int *lwork, real *rwork, int *info);
+#else
+extern void FR(geev,GEEV) (char *jobvl, char *jobvr, int *n,
+                           scalar *A, int *lda, real *wr, real *wi,
+                           scalar *VL, int *ldvl, scalar *VR, int *ldvr,
+                           scalar *work, int *lwork, int *info);
+#endif
+
+#ifdef __cplusplus0
 }                               /* extern "C" */
 #endif                          /* __cplusplus */
 
@@ -143,7 +157,7 @@ void blasglue_rscal(int n, real a, scalar *x, int incx)
 
 void blasglue_copy(int n, scalar *x, int incx, scalar *y, int incy)
 {
-     F(copy,COPY) (&n, x, &incx, y, &incy);
+    F(copy,COPY) (&n, x, &incx, y, &incy);
 }
 
 scalar blasglue_dotc(int n, scalar *x, int incx, scalar *y, int incy)
@@ -190,7 +204,7 @@ void blasglue_gemm(char transa, char transb, int m, int n, int k,
      }
 
      CHECK(A != C && B != C, "gemm output array must be distinct");
-     
+
      ASSIGN_REAL(alpha,a);
      ASSIGN_REAL(beta,b);
 
@@ -214,7 +228,7 @@ void blasglue_herk(char uplo, char trans, int n, int k,
      }
 
      CHECK(A != C, "herk output array must be distinct");
-     
+
      uplo = uplo == 'U' ? 'L' : 'U';
      trans = (trans == 'C' || trans == 'T') ? 'N' : 'C';
 
@@ -289,7 +303,7 @@ int lapackglue_hetri(char uplo, int n, scalar *A, int fdA,
      return (info == 0);
 }
 
-void lapackglue_heev(char jobz, char uplo, int n, scalar *A, int fdA, 
+void lapackglue_heev(char jobz, char uplo, int n, scalar *A, int fdA,
 		     real *w, scalar *work, int lwork, real *rwork)
 {
      int info;
@@ -307,7 +321,52 @@ void lapackglue_heev(char jobz, char uplo, int n, scalar *A, int fdA,
      CHECK(info <= 0, "failure to converge in heev");
 }
 
-void lapackglue_syev(char jobz, char uplo, int n, real *A, int fdA, 
+void lapackglue_geev(char jobvl, char jobvr, int n,
+                     scalar *A, int fdA, scalar_complex *w,
+                     scalar *VL, int fdVL, scalar *VR, int fdVR,
+		     scalar *work, int lwork, real *rwork)
+{
+     int info;
+
+#ifdef SCALAR_COMPLEX
+     F(geev,GEEV) (&jobvl, &jobvr, &n, A, &fdA, w, VL, &fdVL, VR, &fdVR,
+                   work, &lwork, rwork, &info);
+#else
+     int i;
+     real *wr, *wi;
+     CHK_MALLOC(wr, real, 2*n);
+     wi = wr + n;
+     (void) rwork; /* unused */
+     FR(geev,GEEV) (&jobvl, &jobvr, &n, A, &fdA, wr, wi, VL, &fdVL, VR, &fdVR,
+                   work, &lwork, &info);
+     for (i = 0; i < n; ++i)
+         CASSIGN_SCALAR(w[i], wr[i], wi[i]);
+     free(wr);
+#endif
+     CHECK(info >= 0, "invalid argument in geev");
+     CHECK(info <= 0, "failure to converge in geev");
+}
+
+void lapackglue_hegv(int itype, char jobz, char uplo, int n,
+                     scalar *A, int fdA, scalar *B, int fdB,
+		     real *w, scalar *work, int lwork, real *rwork)
+{
+     int info;
+
+     uplo = uplo == 'U' ? 'L' : 'U';
+
+#ifdef SCALAR_COMPLEX
+     F(hegv,HEGV) (&itype, &jobz, &uplo, &n, A, &fdA, B, &fdB, w, work, &lwork, rwork, &info);
+#else
+     (void) rwork; /* unused */
+     F(sygv,SYGV) (&itype, &jobz, &uplo, &n, A, &fdA, B, &fdB, w, work, &lwork, &info);
+#endif
+
+     CHECK(info >= 0, "invalid argument in hegv");
+     CHECK(info <= 0, "failure to converge in hegv");
+}
+
+void lapackglue_syev(char jobz, char uplo, int n, real *A, int fdA,
 		     real *w, real *work, int lwork)
 {
      int info;
