@@ -163,7 +163,31 @@ void evectmatrix_XtX(sqmatrix U, evectmatrix X, sqmatrix S)
      }
 
      mpi_allreduce(S.data, U.data, U.p * U.p * SCALAR_NUMVALS,
-		   real, SCALAR_MPI_TYPE, MPI_SUM, MPI_COMM_WORLD);
+		   real, SCALAR_MPI_TYPE, MPI_SUM, mpb_comm);
+}
+
+/* Dot p selected columns of X with q in Y, starting at ix and iy.
+   Stores the result in U, starting at column iu, with 
+   S1 and S2 as scratch matrices. */
+void evectmatrix_XtY_slice2(sqmatrix U, evectmatrix X, evectmatrix Y,
+                            int ix, int iy, int p, int q, int iu,
+                            sqmatrix S1, sqmatrix S2)
+{
+    int i, j;
+    CHECK(ix + p <= X.p && iy + q <= Y.p && ix >= 0 && iy >= 0 && X.n == Y.n
+          && p == U.p && q <= p && p <= S1.alloc_p && p <= S2.alloc_p, "invalid arguments to XtY_slice2");
+    
+    memset(S1.data, 0, sizeof(scalar) * (U.p * U.p));
+    blasglue_gemm('C', 'N', p, q, X.n,
+                  1.0, X.data + ix, X.p, Y.data + iy, Y.p, 0.0,
+                  S1.data, q);
+    evectmatrix_flops += X.N * X.c * q * (2*p);
+    
+    mpi_allreduce(S1.data, S2.data, p * q * SCALAR_NUMVALS,
+                  real, SCALAR_MPI_TYPE, MPI_SUM, mpb_comm);
+    for (i = 0; i < p; ++i)
+        for (j = 0; j < q; ++j)
+            U.data[i*p + j + iu] = S2.data[i*q + j];
 }
 
 /* Dot p selected columns of X with those in Y, starting at ix and iy.
@@ -171,16 +195,7 @@ void evectmatrix_XtX(sqmatrix U, evectmatrix X, sqmatrix S)
 void evectmatrix_XtY_slice(sqmatrix U, evectmatrix X, evectmatrix Y,
 			   int ix, int iy, int p, sqmatrix S)
 {
-     CHECK(ix + p <= X.p && iy + p <= Y.p && ix >= 0 && iy >= 0 && X.n == Y.n
-           && p == U.p && p <= S.alloc_p, "invalid arguments to XtY_slice");
-
-     memset(S.data, 0, sizeof(scalar) * (U.p * U.p));
-     blasglue_gemm('C', 'N', p, p, X.n,
-                   1.0, X.data + ix, X.p, Y.data + iy, Y.p, 0.0, S.data, U.p);
-     evectmatrix_flops += X.N * X.c * p * (2*p);
-
-     mpi_allreduce(S.data, U.data, U.p * U.p * SCALAR_NUMVALS,
-                   real, SCALAR_MPI_TYPE, MPI_SUM, MPI_COMM_WORLD);
+    evectmatrix_XtY_slice2(U, X, Y, ix, iy, p, p, 0, S, U);
 }
 
 /* compute U = adjoint(X) * Y, with S a scratch matrix. */
@@ -212,7 +227,7 @@ void evectmatrixXtY_sub(sqmatrix U, int Uoffset, evectmatrix X, evectmatrix Y,
      for (i = 0; i < Y.p; ++i) {
 	  mpi_allreduce(S.data + i*Y.p, U.data + Uoffset + i*U.p, 
 			Y.p * SCALAR_NUMVALS,
-			real, SCALAR_MPI_TYPE, MPI_SUM, MPI_COMM_WORLD);
+			real, SCALAR_MPI_TYPE, MPI_SUM, mpb_comm);
      }
 }
 
@@ -224,7 +239,7 @@ void evectmatrix_XtY_diag(evectmatrix X, evectmatrix Y, scalar *diag,
      matrix_XtY_diag(X.data, Y.data, X.n, X.p, scratch_diag);
      evectmatrix_flops += X.N * X.c * X.p * 2;
      mpi_allreduce(scratch_diag, diag, X.p * SCALAR_NUMVALS, 
-		   real, SCALAR_MPI_TYPE, MPI_SUM, MPI_COMM_WORLD);
+		   real, SCALAR_MPI_TYPE, MPI_SUM, mpb_comm);
 }
 
 /* As above, but only compute real parts of diagonal. */
@@ -234,7 +249,7 @@ void evectmatrix_XtY_diag_real(evectmatrix X, evectmatrix Y, real *diag,
      matrix_XtY_diag_real(X.data, Y.data, X.n, X.p, scratch_diag);
      evectmatrix_flops += X.N * X.c * X.p * (2*X.p);
      mpi_allreduce(scratch_diag, diag, X.p,
-		   real, SCALAR_MPI_TYPE, MPI_SUM, MPI_COMM_WORLD);
+		   real, SCALAR_MPI_TYPE, MPI_SUM, mpb_comm);
 }
 
 /* As above, but compute only the diagonal elements of XtX. */
@@ -243,7 +258,7 @@ void evectmatrix_XtX_diag_real(evectmatrix X, real *diag, real *scratch_diag)
      matrix_XtX_diag_real(X.data, X.n, X.p, scratch_diag);
      evectmatrix_flops += X.N * X.c * X.p * (2*X.p);
      mpi_allreduce(scratch_diag, diag, X.p,
-		   real, SCALAR_MPI_TYPE, MPI_SUM, MPI_COMM_WORLD);
+		   real, SCALAR_MPI_TYPE, MPI_SUM, mpb_comm);
 }
 
 /* compute trace(adjoint(X) * Y) */
@@ -257,7 +272,7 @@ scalar evectmatrix_traceXtY(evectmatrix X, evectmatrix Y)
      evectmatrix_flops += X.N * X.c * X.p * (2*X.p) + X.p;
 
      mpi_allreduce(&trace_scratch, &trace, SCALAR_NUMVALS,
-		   real, SCALAR_MPI_TYPE, MPI_SUM, MPI_COMM_WORLD);
+		   real, SCALAR_MPI_TYPE, MPI_SUM, mpb_comm);
 
      return trace;
 }
