@@ -83,6 +83,9 @@ extern void evectmatrix_XtY(sqmatrix U, evectmatrix X, evectmatrix Y,
 			    sqmatrix S);
 extern void evectmatrix_XtY_slice(sqmatrix U, evectmatrix X, evectmatrix Y,
 				  int ix, int iy, int p, sqmatrix S);
+extern void evectmatrix_XtY_slice2(sqmatrix U, evectmatrix X, evectmatrix Y,
+                                   int ix, int iy, int p, int q, int iu,
+                                   sqmatrix S1, sqmatrix S2);
 extern void evectmatrixXtY_sub(sqmatrix U, int Uoffset,
 			       evectmatrix X, evectmatrix Y, sqmatrix S);
 extern void evectmatrix_XtY_diag(evectmatrix X, evectmatrix Y, scalar *diag,
@@ -111,6 +114,8 @@ extern void sqmatrix_aApbB(real a, sqmatrix A, real b, sqmatrix B);
 extern int sqmatrix_invert(sqmatrix U, short positive_definite,
 			    sqmatrix Work);
 extern void sqmatrix_eigensolve(sqmatrix U, real *eigenvals, sqmatrix W);
+extern void sqmatrix_gen_eigensolve(sqmatrix U, sqmatrix B, real *eigenvals, sqmatrix W);
+extern void sqmatrix_eigenvalues(sqmatrix A, scalar_complex *eigenvals);
 extern void sqmatrix_sqrt(sqmatrix Usqrt, sqmatrix U, sqmatrix W);
 
 #ifdef __cplusplus
