@@ -61,7 +61,6 @@ void sqmatrix_assert_hermitian(sqmatrix A)
 void sqmatrix_copy(sqmatrix A, sqmatrix B)
 {
      CHECK(A.p == B.p, "arrays not conformant");
-
      blasglue_copy(A.p * A.p, B.data, 1, A.data, 1);
 }
 
@@ -245,32 +244,74 @@ int sqmatrix_invert(sqmatrix U, short positive_definite,
      return 1;
 }
 
-/* U <- eigenvectors of U.  U must be Hermitian. eigenvals <- eigenvalues.
-   W is a work array.  The columns of adjoint(U') are the eigenvectors, so that
-   U == adjoint(U') D U', with D = diag(eigenvals). 
+/* U <- eigenvectors of Ux=lambda B x, while B is overwritten (by its
+   Cholesky factors).  U and B must be Hermitian, and B must be
+   positive-definite; if B==NULL then it is taken to be the
+   identity. eigenvals <- eigenvalues.  W is a work array.  The
+   columns of adjoint(U') are the eigenvectors, so that U ==
+   adjoint(U') D U', with D = diag(eigenvals).
 
    The eigenvalues are returned in ascending order. */
-void sqmatrix_eigensolve(sqmatrix U, real *eigenvals, sqmatrix W)
+void sqmatrix_gen_eigensolve(sqmatrix U, sqmatrix B, real *eigenvals, sqmatrix W)
 {
      real *work;
+     scalar *morework;
+     int nwork;
 
      sqmatrix_assert_hermitian(U);
      CHK_MALLOC(work, real, 3*U.p - 2);
-
-     if (W.p * W.p >= 3 * U.p - 1)
-	  lapackglue_heev('V', 'U', U.p, U.data, U.p, eigenvals,
-			  W.data, W.p * W.p, work);
+     if (W.p * W.p >= 3 * U.p - 1) {
+         morework = W.data;
+         nwork = W.p * W.p;
+     }
      else {
-	  scalar *morework;
-	  CHK_MALLOC(morework, scalar, 3 * U.p - 1);
-	  lapackglue_heev('V', 'U', U.p, U.data, U.p, eigenvals,
-			  morework, 3 * U.p - 1, work);
-	  free(morework);
+         CHK_MALLOC(morework, scalar, 3 * U.p - 1);
+         nwork = 3 * U.p - 1;
+     }
+     if (B.data) {
+         CHECK(B.p == U.p, "mismatched matrix sizes in sqmatrix_eigensolve");
+         sqmatrix_assert_hermitian(B);
+         lapackglue_hegv(1, 'V', 'U', U.p, U.data, B.p, B.data, U.p, eigenvals,
+                         morework, nwork, work);
+     }
+     else {
+         lapackglue_heev('V', 'U', U.p, U.data, U.p, eigenvals,
+                         morework, nwork, work);
      }
 
+     if (morework != W.data) free(morework);
      free(work);
 }
 
+void sqmatrix_eigensolve(sqmatrix U, real *eigenvals, sqmatrix W)
+{
+    sqmatrix B;
+    B.data = NULL;
+    sqmatrix_gen_eigensolve(U, B, eigenvals, W);
+}
+
+/* Compute eigenvalues of a general (non-Hermitian) matrix A.
+   Does not compute the eigenvectors. */
+void sqmatrix_eigenvalues(sqmatrix A, scalar_complex *eigvals)
+{
+    sqmatrix B; /* make a copy of A, since geev overwrites array */
+    scalar *work, work1;
+    real *rwork;
+    int lwork;
+    B = create_sqmatrix(A.p);
+    sqmatrix_copy(B, A);
+    CHK_MALLOC(rwork, real, 2*A.p);
+    lapackglue_geev('N','N', A.p, B.data, A.p, eigvals, NULL,1,NULL,1,
+                    &work1, -1, rwork);
+    lwork = (int) (SCALAR_RE(work1) + 0.5);
+    CHK_MALLOC(work, scalar, lwork);
+    lapackglue_geev('N','N', A.p, B.data, A.p, eigvals, NULL,1,NULL,1,
+                    work, lwork, rwork);
+    free(work);
+    free(rwork);
+    destroy_sqmatrix(B);
+}
+
 /* Compute Usqrt <- sqrt(U), where U must be Hermitian positive-definite. 
    W is a work array, and must be the same size as U.  Both U and
    W are overwritten. */
