@@ -136,7 +136,7 @@ maxwell_data *create_maxwell_data(int nx, int ny, int nz,
 #    endif
 
      fft_data_size = *alloc_N 
-	  = FFTW(mpi_local_size_transposed)(rank, np, MPI_COMM_WORLD,
+	  = FFTW(mpi_local_size_transposed)(rank, np, mpb_comm,
 					    &local_nx, &local_x_start,
 					    &local_ny, &local_y_start);
 #    ifndef SCALAR_COMPLEX
@@ -158,13 +158,13 @@ maxwell_data *create_maxwell_data(int nx, int ny, int nz,
      CHECK(rank > 1, "rank < 2 MPI computations are not supported");
 
 #    ifdef SCALAR_COMPLEX
-     d->iplans[0] = fftwnd_mpi_create_plan(MPI_COMM_WORLD, rank, n,
+     d->iplans[0] = fftwnd_mpi_create_plan(mpb_comm, rank, n,
 				       FFTW_FORWARD,
 				       FFTW_ESTIMATE | FFTW_IN_PLACE);
      {
 	  int nt[3]; /* transposed dimensions for reverse FFT */
 	  nt[0] = n[1]; nt[1] = n[0]; nt[2] = n[2]; 
-	  d->plans[0] = fftwnd_mpi_create_plan(MPI_COMM_WORLD, rank, nt,
+	  d->plans[0] = fftwnd_mpi_create_plan(mpb_comm, rank, nt,
 					   FFTW_BACKWARD,
 					   FFTW_ESTIMATE | FFTW_IN_PLACE);
      }
@@ -179,7 +179,7 @@ maxwell_data *create_maxwell_data(int nx, int ny, int nz,
 
      CHECK(rank > 1, "rank < 2 MPI computations are not supported");
 
-     d->iplans[0] = rfftwnd_mpi_create_plan(MPI_COMM_WORLD, rank, n,
+     d->iplans[0] = rfftwnd_mpi_create_plan(mpb_comm, rank, n,
 					FFTW_REAL_TO_COMPLEX,
 					FFTW_ESTIMATE | FFTW_IN_PLACE);
 
@@ -187,7 +187,7 @@ maxwell_data *create_maxwell_data(int nx, int ny, int nz,
 	the reverse transform here--we always pass the dimensions of the
 	original real array, and rfftwnd_mpi assumes that if one
 	transform is transposed, then the other is as well. */
-     d->plans[0] = rfftwnd_mpi_create_plan(MPI_COMM_WORLD, rank, n,
+     d->plans[0] = rfftwnd_mpi_create_plan(mpb_comm, rank, n,
 				       FFTW_COMPLEX_TO_REAL,
 				       FFTW_ESTIMATE | FFTW_IN_PLACE);
 
@@ -218,6 +218,7 @@ maxwell_data *create_maxwell_data(int nx, int ny, int nz,
 #endif
 
      CHK_MALLOC(d->eps_inv, symmetric_matrix, d->fft_output_size);
+     d->mu_inv = NULL;
 
      /* A scratch output array is required because the "ordinary" arrays
 	are not in a cartesian basis (or even a constant basis). */
@@ -235,6 +236,7 @@ maxwell_data *create_maxwell_data(int nx, int ny, int nz,
      CHK_MALLOC(d->k_plus_G_normsqr, real, *local_N);
 
      d->eps_inv_mean = 1.0;
+     d->mu_inv_mean = 1.0;
 
      d->local_N = *local_N;
      d->N_start = *N_start;
@@ -275,6 +277,7 @@ void destroy_maxwell_data(maxwell_data *d)
 	  }
 
 	  free(d->eps_inv);
+          if (d->mu_inv) free(d->mu_inv);
 #if defined(HAVE_FFTW3)
 	  FFTW(free)(d->fft_data);
 	  if (d->fft_data2 != d->fft_data)
@@ -305,6 +308,52 @@ static void compute_cross(real *a0, real *a1, real *a2,
      *a2 = b0 * c1 - b1 * c0;
 }
 
+// kdom is returned in Cartesian coordinates
+void maxwell_dominant_planewave(maxwell_data *d, evectmatrix H, int band, double kdom[3])
+{
+    double max_amp = 0;
+    int max_i = 0, i = 0;
+
+    CHECK(d != NULL, "maxwell_data is NULL");
+    CHECK(1 <= band && band <= H.p, "band out of range");
+
+    /* find biggest planewave component in band 'band' of H on current process: */
+    for (i = 0; i < H.localN; ++i) {
+        double amp = SCALAR_NORMSQR(H.data[(i*2+0)*H.p + band-1]) + SCALAR_NORMSQR(H.data[(i*2+1)*H.p + band-1]);
+        if (amp > max_amp) {
+            max_amp = amp;
+            max_i = i;
+        }
+    }
+
+    k_data cur_k = d->k_plus_G[max_i];
+
+    /* set kdom to cur_k.kmag * (cross product of cur_k.m and cur_k.n) */
+    compute_cross(&kdom[0], &kdom[1], &kdom[2],
+                  cur_k.mx, cur_k.my, cur_k.mz,
+                  cur_k.nx, cur_k.ny, cur_k.nz);
+    kdom[0] *= cur_k.kmag;
+    kdom[1] *= cur_k.kmag;
+    kdom[2] *= cur_k.kmag;
+
+#ifdef HAVE_MPI
+    struct {
+        double amp;
+        int rank;
+    } local_res, global_res;
+
+    /* find kdom of biggest max_amp across all processes */
+    local_res.amp = max_amp;
+    local_res.rank = my_global_rank();
+
+    global_res.amp = 0;
+    global_res.rank = 0;
+
+    MPI_Allreduce(&local_res, &global_res, 1, MPI_DOUBLE_INT, MPI_MAXLOC, mpb_comm);
+    MPI_Bcast(kdom, 3, MPI_DOUBLE, global_res.rank, mpb_comm);
+#endif
+}
+
 /* Set the current k point for the Maxwell solver.  k is given in the
    basis of the reciprocal lattice vectors, G1, G2, and G3. */
 void update_maxwell_data_k(maxwell_data *d, real k[3],
