@@ -103,13 +103,15 @@ typedef struct {
      int nplans, plans_howmany[MAX_NPLANS], plans_stride[MAX_NPLANS], plans_dist[MAX_NPLANS];
 
      scalar *fft_data, *fft_data2;
-     
+
      int zero_k;  /* non-zero if k is zero (handled specially) */
      k_data *k_plus_G;
      real *k_plus_G_normsqr;
 
      symmetric_matrix *eps_inv;
      real eps_inv_mean;
+     symmetric_matrix *mu_inv;
+     real mu_inv_mean;
 } maxwell_data;
 
 extern maxwell_data *create_maxwell_data(int nx, int ny, int nz,
@@ -121,6 +123,8 @@ extern void destroy_maxwell_data(maxwell_data *d);
 
 extern void maxwell_set_num_bands(maxwell_data *d, int num_bands);
 
+extern void maxwell_dominant_planewave(maxwell_data *d, evectmatrix H, int band, double kdom[3]);
+
 extern void update_maxwell_data_k(maxwell_data *d, real k[3],
 				  real G1[3], real G2[3], real G3[3]);
 
@@ -145,15 +149,22 @@ extern void set_maxwell_dielectric(maxwell_data *md,
 				   maxwell_dielectric_mean_function mepsilon,
 				   void *epsilon_data);
 
+extern void set_maxwell_mu(maxwell_data *md,
+                           const int mesh_size[3],
+                           real R[3][3], real G[3][3],
+                           maxwell_dielectric_function mu,
+                           maxwell_dielectric_mean_function mmu,
+                           void *mu_data);
+
 extern void maxwell_sym_matrix_eigs(real eigs[3], const symmetric_matrix *V);
 extern void maxwell_sym_matrix_invert(symmetric_matrix *Vinv,
                                       const symmetric_matrix *V);
 extern void maxwell_sym_matrix_rotate(symmetric_matrix *RAR,
 				      const symmetric_matrix *A_,
-				      const double R[3][3]);
+				      double R[3][3]);
 extern int maxwell_sym_matrix_positive_definite(symmetric_matrix *V);
 
-extern void maxwell_compute_fft(int dir, maxwell_data *d, 
+extern void maxwell_compute_fft(int dir, maxwell_data *d,
 				scalar *array_in, scalar *array_out,
 				int howmany, int stride, int dist);
 extern void maxwell_compute_d_from_H(maxwell_data *d, evectmatrix Xin,
@@ -162,6 +173,10 @@ extern void maxwell_compute_d_from_H(maxwell_data *d, evectmatrix Xin,
 extern void maxwell_compute_h_from_H(maxwell_data *d, evectmatrix Hin,
 				     scalar_complex *hfield,
 				     int cur_band_start, int cur_num_bands);
+extern void maxwell_compute_H_from_B(maxwell_data *d, evectmatrix Bin,
+                                     evectmatrix Hout, scalar_complex *hfield,
+                                     int Bin_band_start, int Hout_band_start,
+                                     int cur_num_bands);
 extern void maxwell_compute_e_from_d(maxwell_data *d,
 				     scalar_complex *dfield,
 				     int cur_num_bands);
@@ -169,9 +184,9 @@ extern void maxwell_compute_e_from_d(maxwell_data *d,
 extern void maxwell_vectorfield_otherhalf(maxwell_data *d,
 					  scalar_complex *field,
 					  real phasex,real phasey,real phasez);
-extern void maxwell_cscalarfield_otherhalf(maxwell_data *d, 
+extern void maxwell_cscalarfield_otherhalf(maxwell_data *d,
 					   scalar_complex *field,
-					   real phasex, real phasey, 
+					   real phasex, real phasey,
 					   real phasez);
 extern void maxwell_scalarfield_otherhalf(maxwell_data *d, real *field);
 
@@ -181,6 +196,8 @@ void assign_symmatrix_vector(scalar_complex *newv,
 
 extern void maxwell_operator(evectmatrix Xin, evectmatrix Xout, void *data,
 			     int is_current_eigenvector, evectmatrix Work);
+extern void maxwell_muinv_operator(evectmatrix Xin, evectmatrix Xout, void *data,
+                                   int is_current_eigenvector, evectmatrix Work);
 extern void maxwell_simple_precondition(evectmatrix X,
 					void *data, real *eigenvals);
 extern void maxwell_preconditioner(evectmatrix Xin, evectmatrix Xout,
