@@ -433,7 +433,7 @@ int main(int argc, char **argv)
      printf("\nSolving for eigenvectors with preconditioning...\n");
      evectmatrix_copy(H, Hstart);
      eigensolver(H, eigvals,
-		 op, op_data,
+		 op, op_data, NULL,NULL,
 		 pre_op, pre_op_data,
 		 maxwell_parity_constraint, (void *) mdata,
 		 W, NWORK, error_tol, &num_iters, EIGS_DEFAULT_FLAGS);
@@ -457,6 +457,21 @@ int main(int argc, char **argv)
      }
      printf("\n");
 
+     for (i = 0; i < num_bands; ++i) {
+         real kdom[3];
+         real k;
+         maxwell_dominant_planewave(mdata, H, i + 1, kdom);
+         if ((i + 1) % 2 == 1)
+             k = kvector[0] + (i + 1) / 2;
+         else
+             k = kvector[0] - (i + 1) / 2;
+         if (kvector[0] > 0 && kvector[0] < 0.5 && ed.eps_high == 1) {
+             printf("Expected kdom: %15f%15f%15f\n", k, kvector[1], kvector[2]);
+             printf("Got kdom:      %15f%15f%15f\n", kdom[0], kdom[1], kdom[2]);
+             CHECK(k == kdom[0] && kvector[1] == kdom[1] && kvector[2] == kdom[2],
+                   "unexpected result from maxwell_dominant_planewave");
+         }
+     }
      }
 
      if (!stop1) {
@@ -466,7 +481,7 @@ int main(int argc, char **argv)
      printf("\nSolving for eigenvectors without preconditioning...\n");
      evectmatrix_copy(H, Hstart);
      eigensolver(H, eigvals,
-		 op, op_data,
+		 op, op_data, NULL,NULL,
 		 NULL, NULL,
 		 maxwell_parity_constraint, (void *) mdata,
 		 W, NWORK, error_tol, &num_iters, EIGS_DEFAULT_FLAGS);
@@ -495,7 +510,7 @@ int main(int argc, char **argv)
      printf("\nSolving for eigenvectors without conj. grad...\n");
      evectmatrix_copy(H, Hstart);
      eigensolver(H, eigvals,
-		 op, op_data,
+		 op, op_data, NULL,NULL,
 		 pre_op, pre_op_data,
 		 maxwell_parity_constraint, (void *) mdata,
 		 W, NWORK - 1, error_tol, &num_iters, EIGS_DEFAULT_FLAGS);
@@ -524,7 +539,7 @@ int main(int argc, char **argv)
      evectmatrix_copy(H, Hstart);
      eigensolver(H, eigvals,
 		 op, op_data,
-		 NULL, NULL,
+		 NULL, NULL, NULL,NULL,
 		 maxwell_parity_constraint, (void *) mdata,
 		 W, NWORK - 1, error_tol, &num_iters, EIGS_DEFAULT_FLAGS);
 
